<?php
// This file is part of
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


class Teacherview {
  /**
   * @property gradeimporterid used to query DB at get_submissions()
   * @property context used to get studentslist at get_studentlist()
   * @property cellstyle style used by table cells. Black border and centered text
   */
  private $gradeimporterid;
  private $context;
  private $cmid;
  private $cellstyle = "border:1px solid black; text-align:center";
  private $edit = 0;

  /**
   * @param int cmid to get context
   * @param int gradeimporterid to query database
   * constructor
   */
  public function __construct (int $cmid, int $gradeimporterid, int $edit) {
    $this->gradeimporterid = $gradeimporterid;
    $this->cmid = $cmid;
    $this->context = context_module::instance($cmid);
    $this->edit = $edit;
  }

  private function get_gradeimporterid () {
    return $this->gradeimporterid;
  }
  private function get_cmid () {
    return $this->cmid;
  }
  private function get_context () {
    return $this->context;
  }
  private function get_cellstyle ($extrastyle = "") {
    return "$this->cellstyle $extrastyle";
  }

  private function get_edit () {
    return $this->edit;
  }

  /**
   * builds teacherview table using php html_table class
   * Gets all submissions from get_submissions()
   * Header is built with get_table_head() passing submissions as param
   * Each row is a different student
   * Gets each row with get_studentsubmissions()
   * @return table as html string
   */
  public function make_table () {
    global $DB, $CFG;

    // Prepare variables
    $studentlist = $this->get_studentlist();
    $submissions = $this->get_submissions();

    // Create table
    $table = new html_table();
    $table->align = array('center');
    $table->attributes = array('class' => 'generaltable mod_index');

    $submissions = $this->get_submissions();
    if (!$submissions) {
      $this->no_submissions();
      return;
    }
    $table->data[] = $this->get_table_head($submissions);
    foreach ($studentlist as $student) {
      $table->data[] = $this->get_studentsubmissions($student, $submissions);
    }

    return html_writer::table($table);
  }

  /**
   * Gets students list from moodle function get_enrolled_users
   * Builds a list from $enrolledusers with:
   * * Keys as student id;
   * * Values as student fullname and student id.
   * @return array $studentlist - students data (name and id)
   */
  private function get_studentlist () {
    // Get students list with userid as key and fullname as value
    $enrolledusers = get_enrolled_users($this->get_context(), 'mod/gradeimporter:view',
                    0, 'u.id, u.firstname, u.lastname',
                    'u.firstname, u.lastname'
                  );
    $studentlist = array();
    foreach ($enrolledusers as $user) {
      $studentlist[$user->id] = array('name' => $user->firstname." ".$user->lastname,
                      'id' => $user->id);
    }

    return $studentlist;
  }

  /**
   * Builds a row with first column being "name" for students names
   * Other rows are submissions names
   * @return html_table_row $header
   */
  private function get_table_head ($submissions) {
    // Creates teacher view table head
    $header = new html_table_row();

    // Add name header to header row
    $title = get_string('nameColTitle', 'gradeimporter');
    $text = get_string('nameCol', 'gradeimporter');
    $celltext = "<a title = \"$title\"> $text </a>";
    $header->cells[] = $this->make_cell($celltext, 1);

    foreach ($submissions as $submission) {
      // Add a icon to redirect to submission form to edit the sub
      $celltext = $this->subcelltext($submission);
      $header->cells[] = $this->make_cell($celltext, 2);
    }
    return $header;
  }

  /**
   * Build a row for a student defined by paramcmide and link to " - "
   * @return html_table_row $row with each column being grade and feedbackfile for the submission
   */
  private function get_studentsubmissions ($student, $submissions) {
    global $DB;
    // Create new row for the student
    $row = new html_table_row();

    // Set first cell of the row as the students fullname
    $row->cells[] = $this->make_cell($student["name"], 1);

    // Foreach submission checks if student has a feedback for it
    // If they have, fill it with grade + filename (possibly grade+link to full feedback)
    // If they don't have fill it with grade and filename as -
    foreach ($submissions as $submission) {
      $feedback = $DB->get_record('gradeimporter_feedback',
                    ['submissionid' => $submission->id,
                      'studentid' => $student['id']]
                    );
      if ($feedback) {
        $fileurl = $this->feedback_url($feedback);
        $grade = $feedback->grade;
      } else {
        $fileurl = '-';
        $grade = '-';
      }
      $row->cells[] = $this->make_cell($grade, 1);

      $row->cells[] = $this->make_cell($fileurl, 1);
    }
    return $row;
  }

  /**
   * Fetch all submissions for this gradeimporter instance on gradeimporter_submissions table
   * Build query using $this->gradeimporterid property
   * @return $submissions fetched from gradeimporter_submission table
   */
  private function get_submissions () {
    global $CFG, $DB;
    // Get table prefix
    $tp = $CFG->prefix;

    // Build Query
    $sql = "SELECT id, name, type, intro
        FROM {$tp}gradeimporter_submission
        WHERE gradeimporterid ={$this->get_gradeimporterid()}
        ORDER BY type, id";
    // Query DB
    $submissions = $DB->get_records_sql($sql);
    // Return submissions
    return $submissions;
  }

  /**
   * Prints html when there are no submissions to show
   * @underconstruction
   * @return void
   */
  private function no_submissions () {
    echo "No submissions";
  }

  /**
   * Builds fileplugin url to feedback file
   * @param array $feedback fetched from gradeimporter_feedback table
   * @return html hyperlink to download feedback file
   */
  private function feedback_url ($feedback) {
    $url = moodle_url::make_pluginfile_url($feedback->contextid,
                      'mod_gradeimporter',
                      'submissionfiles',
                      $feedback->id,
                      "/",
                      $feedback->filename,
                      true
                    );
    return "<a href='{$url}'>$feedback->filename</a>";
  }

  /**
   * Makes a cell object to be inserted into a row
   * Cell style is predifined on class properties
   * @param string $text - Text shown on the cell
   * @param int $colspan - How many columns this cell ocuppies
   * @return html_table_cell $cell - cell built on the function
   */
  private function make_cell ($text, $colspan) {
    $cell = new html_table_cell($text);
    $cell->style = $this->get_cellstyle();
    $cell->colspan = $colspan;
    return $cell;
  }

  private function subcelltext ($submission) {

    // Base celltext, shown when editing or not
    $celltext = "<a title='$submission->intro'>
                    $submission->name
                </a>";

    // If is editing submission, add a edit icons
    if ($this->get_edit()) {

      // Build edit link icon + url
      $editurl = new moodle_url("/mod/gradeimporter/forms/submission/submission.php", array('id' => $this->get_gradeimporterid(),
                                                                                      'cmid' => $this->get_cmid(),
                                                                                      'subid' => $submission->id)
                                                                                    );
      $editlinktitle = get_string('editSub', 'gradeimporter', $submission->name);
      $celltext .= "  <a href=\"$editurl\">
                        <i class = \"icon fa fa-pencil fa-fw\" title=\"$editlinktitle\" aria-label=\"$editlinktitle\">
                        </i>
                      </a>";

      // Build delete link icon + url
      $deleteurl = new moodle_url("/mod/gradeimporter/forms/submission/delete.php", array('id' => $this->get_gradeimporterid(),
                                                                                      'cmid' => $this->get_cmid(),
                                                                                      'subid' => $submission->id)
                                                                                    );
      $deletelinktitle = get_string('deletesubmission', 'gradeimporter', $submission->name);
      $celltext .= "<a href=\"$deleteurl\">
                      <i class=\"fa fa-trash\" title=\"$deletelinktitle\" aria-label=\"$deletelinktitle\">
                      </i>
                    </a>";
    }
    // Full celltext, with edit and delete icons if teacher is editing
    return $celltext;
  }

}
